#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Tests for casa_envoy.foundation
"""
from pathlib import Path
from unittest.mock import patch

import pytest
from casa_envoy.foundation import (
    CubeFoundation,
    GeneralFoundation,
    RestoreFoundation,
    set_permissions,
)
from casa_envoy.schema import AbstractTextFile

test_general_metadata = AbstractTextFile(
    filename="test/input_files/test-general-foundation.json",
    content=Path("test/input_files/test-general-foundation.json").read_text(),
)

test_restore_metadata = AbstractTextFile(
    filename="test/input_files/restore.json",
    content=Path("test/input_files/restore.json").read_text(),
)


@pytest.fixture
def general(cal_settings) -> GeneralFoundation:
    return GeneralFoundation(parameters=cal_settings, metadata=test_general_metadata)


@pytest.fixture
def restore(cal_settings) -> RestoreFoundation:
    return RestoreFoundation(parameters=cal_settings, metadata=test_restore_metadata)


@pytest.fixture
def cube(cal_settings) -> CubeFoundation:
    return CubeFoundation(parameters=cal_settings)


@patch("os.path.join")
@patch("os.chmod")
@patch("os.walk")
def test_set_permissions(mock_walk, mock_chmod, mock_join, parent_path):
    """
    Verifies reusable set permissions function

    :param mock_walk:
    :param mock_chmod:
    :param mock_join:
    :return:
    """
    set_permissions(parent_path)
    assert mock_walk.call_count == 1


class TestGeneralFoundation:
    """
    Test methods for laying General data foundations
    """

    @patch("shutil.copy")
    def test_general_data_foundation(self, mock_copy, general):
        general.data_foundation()
        assert mock_copy.call_count == 2


class TestRestoreFoundation:
    """
    Test methods for laying Restore data foundations
    """

    @patch("os.chdir")
    def test_restore_data_foundation(self, mock_chdir, restore):
        with patch("casa_envoy.foundation.RestoreFoundation.extract_cal") as extract:
            with patch("casa_envoy.foundation.set_permissions") as permissions:
                restore.data_foundation()
                assert mock_chdir.call_count == 1

    @patch("pathlib.Path.exists", return_value=True)
    @patch("tarfile.open")
    @patch("shutil.copytree")
    def test_extract_cal(self, mock_copytree, mock_tar, mock_path, restore):
        restore.extract_cal()
        assert mock_tar.call_count == 1
        assert mock_copytree.call_count == 1


class TestCubeFoundation:
    """
    Test methods for laying VLASS Cube data foundations
    """

    @patch("pathlib.Path.glob", return_value=["abcatalog.csv", "cdcatalog.csv"])
    @patch("os.chdir")
    def test_cube_data_foundation(self, mock_chdir, mock_glob, cube):
        # with patch("casa_envoy.foundation.CubeFoundation.extract_reimaging_resources") as extract:
        with patch("casa_envoy.foundation.set_permissions") as permissions:
            cube.data_foundation()
            assert mock_chdir.call_count == 1
            # assert extract.call_count == 1
            assert permissions.call_count == 1

    @patch("tarfile.open")
    @patch("pathlib.Path.exists", return_value=True)
    def test_extract_reimaging_resources(self, mock_exists, mock_tar_open, cube):
        cube.extract_reimaging_resources()
        assert mock_exists.call_count == 1
        assert mock_tar_open.call_count == 1

    @patch("pathlib.Path.glob", return_value=["abcatalog.csv", "cdcatalog.csv"])
    @patch("pathlib.Path.exists", return_value=True)
    @patch("shutil.copy")
    def test_move_ancillaries(self, mock_copy, mock_exists, mock_glob, cube, parent_path: Path):
        cube.move_ancillaries()
        assert mock_exists.call_count == 3
        assert mock_copy.call_count == 3
        assert mock_glob.call_count == 1

        # Ensure copy was called on the expected directories
        product_path = parent_path / "products"
        working_dir_path = parent_path / "working"
        mock_copy.assert_any_call("abcatalog.csv", product_path)
        mock_copy.assert_any_call(Path("./rawdata/SEIP_parameter.list"), working_dir_path)
        mock_copy.assert_any_call(Path("./rawdata/reimaging_resources.tgz"), working_dir_path)
